@@ -1,4 +1,4 @@
-#! /usr/bin/python3.3
+#! /usr/bin/python2.7
 
 """Token constants (from "token.h")."""
 
@@ -57,17 +57,18 @@ DOUBLESTAREQUAL = 47
 DOUBLESLASH = 48
 DOUBLESLASHEQUAL = 49
 AT = 50
-OP = 51
-COMMENT = 52
-NL = 53
-RARROW = 54
-ERRORTOKEN = 55
-N_TOKENS = 56
+ATEQUAL = 51
+OP = 52
+COMMENT = 53
+NL = 54
+RARROW = 55
+ERRORTOKEN = 56
+N_TOKENS = 57
 NT_OFFSET = 256
 #--end constants--
 
 tok_name = {}
-for _name, _value in list(globals().items()):
+for _name, _value in globals().items():
     if type(_value) is type(0):
         tok_name[_value] = _name
 
